# My Heats

Live heats, scoring and leaderboard for sport events.

This React application was build using [Vite](https://vitejs.dev) (`npm create vite@latest . --template react`) and the following core componentes:
* [`react-router-dom`](https://reactrouter.com) for routing between the React components
* [`@supabase/supabase-js`](https://supabase.com/docs/guides/getting-started/quickstarts/reactjs) for connection to the Supabase database
* [`react-select`](https://react-select.com) for the selectbox widgets

## Running the App

```bash
# Install dependencies
npm install

# Export Supabase URI and database access key
export VITE_APP_SUPABASE_URL=
export VITE_APP_SUPABASE_KEY=

# Start app for development
npm run dev
```

## Databas Schema

The Supabase schema is stored in the `schema` folder and can be created using plain psql.

![supabase-schema](assets/supabase-schema.png)

To update the schema from the current database, use (example for table `startlist`):
```bash
pg_dump -h db.youruniquedb.supabase.co -U postgres -t 'public.startlist' --schema-only > schema/startlist.sql
```

Additionally, following views and functions are required:

```sql
-- Summarize all scores of a heat
create or replace view score_summary as
  select a.id as athlete_id, s.heat as heat_id, SUM(s.score) as score_summary
  from scores s
  join athletes as a on a.id = s.athlete
  group by a.id, s.heat

-- Create distinct set of athletes (startlist) from a set of heats
create or replace function distinct_startlist(heat_ids numeric[])
returns table(id bigint, athlete bigint, nr bigint, firstname varchar, lastname varchar,
birthday date, school varchar)
language plpgsql
as $$
begin
  return query
  select distinct on (a.id) s.id, a.id, a.nr, a.firstname, a.lastname, a.birthday, a.school
  from startlist as s
  join athletes as a on s.athlete = a.id
  where s.heat = any (heat_ids);
end;
$$;
```

Add a trigger for connecting users with judges:
```sql
-- https://supabase.com/docs/guides/auth/managing-user-data#using-triggers
-- inserts a row into public.judges
create function public.handle_new_judge()
returns trigger
language plpgsql
security definer set search_path = public
as $$
begin
  insert into public.judges (id)
  values (new.id);
  return new;
end;
$$;

-- trigger the function every time a user is created
create trigger on_auth_user_created
  after insert on auth.users
  for each row execute procedure public.handle_new_judge();
```

Lastly, enable [PostgreSQL publication](https://www.postgresql.org/docs/current/logical-replication-publication.html) for the [Supabase realtime feature on Postgres changes](https://supabase.com/docs/guides/realtime/extensions/postgres-changes):
```sql
begin;

-- remove the supabase_realtime publication
drop
  publication if exists supabase_realtime;

-- re-create the supabase_realtime publication with no tables
create publication supabase_realtime;

commit;

-- add a table to the publication
alter
  publication supabase_realtime add table scores;
```

This is required for the leaderboard to automatically update scores when they are created or changed by judges.

## Authentication with Magic Links

Authentication of judges is performed using [Supabase Magic Links](https://supabase.com/docs/guides/auth/auth-magic-link).

The basic code for the authentication flow was taken from the [official React Getting Started Tutorial](https://supabase.com/docs/guides/getting-started/tutorials/with-react).

Sign up process for new judges:
* The judge is required to have a valid email address
* Judges can be invied to the scoring backend by invite only (Click ["Invite"](https://app.supabase.com/project/_/auth/users) in Supabase backend)
* The UUID is referenced automatically in the `judges` table, row can be amended with name of the judge

Sign in process:
* For sign in, the judge enters her email adress and receives a login link
* The login information is stored in the browser session until sign out

## Bulk Import/Export

Use direct postgres database access:
```bash
psql -h db.youruniquedb.supabase.co -U postgres
```

To import data from local csv:
```sql
\copy public.scores from 'scores_rows.csv' DELIMITER ',' CSV HEADER;
```

To export data to local csv:
```sql
\copy public.scores to 'scores_rows.csv' DELIMITER ',' CSV HEADER;
```

## Development & Contributions
* All source code is available in this repository. Contributions are always welcome!
* A [Kanban board](https://board.in0rdr.ch/public/board/c445667f1c999857f6149cad967fa44f196f2e9fef02069c5f5136e036dd) documents plans, todos and decisions for further development of the project.

## Community and Support
For question and support visit [#myheats/libera](https://web.libera.chat/gamja/#myheats) on IRC.

## License
This work uses the MIT License. See [LICENSE](./LICENSE) for details.
