import { lazy, useEffect, useState } from 'react'
import { generatePath, Link } from 'react-router-dom'
import { supabase } from './supabaseClient'

const Auth = lazy(() => import('./Auth'))

async function addHeat(e) {
  e.preventDefault()

  // Read the form data
  const formData = new FormData(e.target);
  const formJson = Object.fromEntries(formData.entries());

  if (defaultsSet(formJson)) {
    alert('Check data of the new heat, seems like the defaults')
    return
  }

  // create new heat
  const { error } = await supabase
    .from('heats')
    .insert({
      name: formJson.name,
      location: formJson.location,
      // planned_start is an empty string if unset
      // https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/time
      planned_start: formJson.planned_start === '' ? null : formJson.planned_start
  })

  if (error === null) {
    window.location.reload()
  } else {
    alert('Failed to create new heat: ' + error.message)
  }
}

export function defaultsSet({name, location}) {
  return (name === 'Name *' || location === 'Location')
}

async function deleteHeat(e, heatId, heatName) {
  e.preventDefault()

  if (window.confirm('Do you really want to delete heat "' + heatName + '"?')) {
    await supabase
      .from('heats')
      .delete()
      .eq('id', heatId)
    window.location.reload()
  }
}

function HeatForm({session}) {
  const [loading, setLoading] = useState(false)
  const [heats, setHeats] = useState([])

  useEffect(() => {
    (async () => {
      setLoading(true)
      const heatList = await supabase.from('heats').select()
      if (heatList.error === null)
        setHeats(heatList.data)
      setLoading(false)
    })();
  }, [])

  return (
    <div>
      <button disabled={!loading}>{loading ? '🔄 loading' : ''}</button>
      <form method='post' onSubmit={addHeat}>
        <table>
          <thead>
            <tr>
              <th>Created at</th>
              <th>Name</th>
              <th>Location</th>
              <th>Planned start</th>
              <th>New/delete</th>
            </tr>
          </thead>
          <tbody>
            {heats.map(h => (
              <tr key={h.id}>
                <td data-title='Created at' className='right'>{new Date(h.created_at).toLocaleString()}</td>
                <td data-title='Name'><Link to={generatePath('/startlist/:heatId', {heatId:h.id})}>{h.name}</Link></td>
                <td data-title='Location'>{h.location}</td>
                <td data-title='Planned start' className='right'>{h.planned_start}</td>
                <td><button onClick={e => deleteHeat(e, h.id, h.name)}>🗑️ delete</button></td>
              </tr>
            ))}
            <tr>
              <td className='right'><i>* required</i></td>
              <td data-title='Name'>
                <input type='text' name='name' defaultValue='Name *' />
              </td>
              <td data-title='Location'>
                <input type='text' name='location' defaultValue='Location' />
              </td>
              <td data-title='Planned start' className='right'>
                <input
                  type='time'
                  name='planned_start' />
              </td>
              <td>
                <button type='submit'>➕ new</button>
              </td>
            </tr>
          </tbody>
        </table>
      </form>
    </div>
  )
}

function Heats({session}) {
    return (
      <div>
        {!session ? <Auth /> : <HeatForm session={session} />}
      </div>
    )
  }
  
export default Heats;
